// 																																			KEDS Project
// 																												(Kansas Event Data System)
//__________________________________________________________________________________
// 										count_mediation: monthly tabulation of third-party mediation 

// This program goes through a data set and counts the number of occasions where a 
// third party engages in cooperative activities with two parties in a conflict.
// These total are output by month.

// Output: Text file "Count_Mediate.out" contains the compilation.  There is no 
//         feedback while the program is running.

//	Programmer: Philip A. Schrodt (schrodt@ku.edu)
//	Compiler:		Metrowerks C
//	Language:		ANSI C except for // comments

//	Revision History: 
//		01.02		Original program -- Modified from HMM "Count_Conflict" program

//__________________________________________________________________________________
// 	Macros

#define	FALSE			0
#define TRUE			1

#define OUT_FILE	"Count_Mediate.out"
#define	MAX_DATE_LAG  7		// maximum number of days to define active mediation

#if TRUE
#define IN_FILE		"Levant.79-99.events"
#define ANTAGO1_LIST "ISR."  					// codes for the two groups of antagonists we are tabulating,
#define ANTAGO2_LIST "LEB."  					// separating by periods to avoid accidental matches
//#define MEDIATR_LIST "USA.UNO.EEC."		// codes for the mediators
#define MEDIATR_LIST "SYR.IRN."		// codes for the mediators
																			// Julian bounds for the data set
#define	HIGH_DAY		34971		//	99.09.30
#define	LOW_DAY   	27497		//	79.04.14
#define FIRST_DAY		"790414"

#else

#define IN_FILE		"BALKANS.events.recoded"
#define ANTAGO1_LIST "SER."  					// codes for the two groups of antagonists we are tabulating,
#define ANTAGO2_LIST "BFR."  					// separating by periods to avoid accidental matches
//#define MEDIATR_LIST "USA.UNO.EEC.NATO"		// codes for the mediators
//#define MEDIATR_LIST "USA."		// codes for the mediators
//#define MEDIATR_LIST "EEC.FRN.GER.ITL.UNK."		// codes for the mediators
//#define MEDIATR_LIST "RUS.UKR."		// codes for the mediators
#define MEDIATR_LIST "USA.NATO.UNK.FRN"		// codes for the use of force group*
																					// Julian bounds for the data set
#define	HIGH_DAY		34849		//	99.07.31
#define	LOW_DAY   	31137		//	89.04.01
#define FIRST_DAY		"890401"
#endif



//___________________________________________________________________________
// Prototypes

long Julian (char *s);	// Converts dates of form YYMMDD to Julian serial beginning 0 = 1 Jan 1904

//__________________________________________________________
// Globals


int		Month[] = {0,0,31,59,90,120,151,181,212,243,273,304,334};		// used for Julian computation: # days prior to Month[n] JAN=1
char  santago1[] = ANTAGO1_LIST;  // codes for antagonist group 1
char  santago2[] = ANTAGO2_LIST;  // codes for antagonist group 2
char  smediatr[] = MEDIATR_LIST;  // codes for mediators
  
//__________________________________________________________________________________

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//__________________________________________________________________________________
//																																			main program

void main(void)
{
FILE	*fin, *fout;// input and output files
int 	ijul;				// julian value of date
int   kmedevt = 0;// number of mediation events
long  last1 = 0;	// julian date of last observed interaction of group
long 	last2 = 0;
char  src[7];			// source and target codes 
char  tar[7];
int   code;				// event code
char 	stail[40];	// read junk from end of input line
char  date[7];		// character version of date of current record
char  curdate[7] = FIRST_DAY;	// character version of date for current month

	fin = fopen(IN_FILE,"r");
	if (!fin) { printf("Error: can't open %s",IN_FILE); exit(0); }
	fout = fopen(OUT_FILE,"w");
	if (!fout) { printf("Error: can't open %s",OUT_FILE); exit(0); }
	fprintf(fout,"Input file: %s\n",IN_FILE);
	fprintf(fout,"Group 1 codes: %s\n",santago1);
	fprintf(fout,"Group 2 codes: %s\n",santago2);
	fprintf(fout,"Mediator codes: %s\n",smediatr);
	fprintf(fout,"Window: %d days\n\n",MAX_DATE_LAG);
	
	while (!feof(fin))
	{
		fscanf(fin,"%s %s %s %d",&date, &src, &tar, &code);

		if (date[3] != curdate[3]) {		// output when we hit a new month
			fprintf(fout,"%2.2s-%2.2s\t%4d\n",curdate,&curdate[2],kmedevt);
			printf("%2.2s-%2.2s\t%4d\n",curdate,&curdate[2],kmedevt);
			strcpy(curdate,date);
			kmedevt = 0;
		}
				
		fgets(stail,40,fin);
		ijul = Julian(date);
		if ((ijul<LOW_DAY) || (ijul>HIGH_DAY))	{				// check this because we don't trust Reuters
			printf("Date error:%s  %d\n",date,ijul);
			strcpy(src,"---");		// insure there will be no match
			continue;
			}

		if ((strstr(santago1,src) && strstr(smediatr,tar)) || 
				(strstr(santago1,tar) && strstr(smediatr,src))) {
			if (((ijul - last2) <= MAX_DATE_LAG ) &&
					(code <=110))
//  					(code >=200))
				 kmedevt++;
			last1 = ijul;
			}
		if ((strstr(santago2,src) && strstr(smediatr,tar)) || 
				(strstr(santago2,tar) && strstr(smediatr,src))) {
			if (((ijul - last1) <= MAX_DATE_LAG ) &&
					(code <=100))
					 kmedevt++;
			last2 = ijul;
			}
	}
	
	fprintf(fout,"%2.2s-%2.2s\t%4d\n",curdate,&curdate[2],kmedevt);
	fclose(fout);
	
}

long Julian (char *s)
// Converts dates of form YYMMDD to Julian serial beginning 0 = 1 Jan 1904
// (which matches the Excel convention) 
{
int 	mo,dy;
long	yr, ij;

  for (mo=0; mo<7; mo++) if (s[mo] == ' ') s[mo] = '0';			// convert blanks to zeros
  
																		//Break date out of the string and convert to integers 
	yr = (short)s[0] * 10 + (short)s[1] - 532;			// 532 substracts the 1904 base from yr
	mo = (short)s[2] * 10 + (short)s[3] - 528;
	dy = (short)s[4] * 10 + (short)s[5] - 528;
						
	ij =  yr * 365;
	ij += ((yr+3) / 4) + Month[mo] + (dy-1);				// ((yr+3)/4) is the leap-year adjustment
	if (((yr % 4) == 0) && (mo > 2)) ij++;	// adjust for month within a leap-year
	return ij;
} // Julian
